<?php

//  Copyright (C) 2005-2008 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

class po_auth_flyspray_db {
  private $handle;
  private $dbprefix = 'flyspray_';
  private $admin_grp = 1;
  private $user_grp = 3;
  private $client_grp = 4;

  function __construct() {    
    global $auth_db_dsn;
    $this->handle = pg_pconnect($auth_db_dsn);

    if ($this->handle)
      pg_set_client_encoding($this->handle, 'UTF-8');
  }

  /* Clean up any resources obtained in the constructor */
  function __destruct() {
    pg_close($this->handle);
  }

  /* Arguments:
   $username = "some_plaintext_username";
   $password = "some_plaintext_password";
   
   Returns:
   FALSE if unsuccessful
   username if successful
  */

  function auth_user($username, $password) {
    $database = $this->handle;
    $password = pg_escape_string($this->passwd_transform($password, $username));
    $username = pg_escape_string($username);
   
    $res = pg_query($database, "SELECT user_name FROM ".$this->dbprefix."users WHERE user_name='$username' and user_pass = '$password' and account_enabled > 0");
    
    if (pg_num_rows($res)) {
      $row = pg_fetch_row($res);
      return $row[0];
    } else {
      return FALSE;
    }
  }

  /* Arguments:
    $username = "some_plaintext_username";

   Returns:  array()

   THESE ARE REQUIRED FIELDS:

    'first_name' => 'firstname'
    'last_name' => 'lastname'
    'email' => 'email_addr'
    'type' => PO_USER_TYPE

    OPTIONAL FIELDS

    address1, address2, city, zipcode, state, country, phone, url

  */
  function user_info($username) {
    $username = pg_escape_string($username);
    
    $res = pg_fetch_assoc(pg_query($this->handle, "SELECT real_name, email_address as email, account_enabled, (select count(*) from ".$this->dbprefix."users_in_groups g where u.user_id = g.user_id and g.group_id = $this->admin_grp) as admin, (select count(*) from ".$this->dbprefix."users_in_groups g where u.user_id = g.user_id and g.group_id = $this->user_grp) as user                             FROM ".$this->dbprefix."users u 
                            WHERE u.user_name='$username' "));
    if ($res['admin'])
      $res['type'] = PO_USER_TYPE_ADMIN;
    elseif ($res['user'])
      $res['type'] = PO_USER_TYPE_USER;
    elseif ($res['account_enabled'])
      $res['type'] = PO_USER_TYPE_CLIENT;
    else
      $res['type'] = PO_USER_TYPE_DISABLED;

    $res['first_name'] = substr($res['real_name'], 0, strpos($res['real_name'], ' '));
    $res['last_name'] = substr($res['real_name'], strpos($res['real_name'], ' ') + 1);

    unset($res['admin']);
    unset($res['client']);
    unset($res['user']);
    unset($res['account_enabled']);
    unset($res['real_name']);

    return $res;
  }
  
  /* Arguments:
    $username = "some_plaintext_username";
    $old_password = "old password, plaintext";
    $new_password = "new password, plaintext";

   Returns:  

     FALSE if failed, something else if ok.
  */
  function change_pass($username, $old_password, $new_password) {
    $database = $this->handle;
    $username = pg_escape_string($username);
    
    $new_password = pg_escape_string($this->passwd_transform($new_password, $username));
    $old_password = pg_escape_string($this->passwd_transform($old_password, $username));
    
    $res = pg_query($database, "update ".$this->dbprefix."users set user_pass = '$new_password' where user_name = '$username' and user_pass = '$old_password'");

    return (pg_affected_rows($res) > 0);
  }

  function force_change_pass($username, $new_password) {
    $database = $this->handle;
    $username = pg_escape_string($username);
    
    $new_password = pg_escape_string($this->passwd_transform($new_password, $username));
    
    $res = pg_query($database, "update ".$this->dbprefix."users set user_pass = '$new_password' where user_name = '$username'");

    return (pg_affected_rows($res) > 0);
  }

  /* Mangles the password for storage in local database */
  function passwd_transform($password, $salt) {
    return md5($password);
  }
  
  public $can_change_pass = TRUE;
  public $username_is_email = FALSE;
  public $local_register = FALSE;
  public $local_userinfo = TRUE;
}

?>
