<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2008 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "config.php";
include_once "mime.php";
include_once "exif.php";
include_once "iptc.php";
include_once "rdf.php";
include_once "site.php";

function count_storage_space_by_user($database, $user_id) {
  $file_sizes = pg_query($database, "
     select sum(filesize) 
       from files, photo_version, photo
      where photo.users = $user_id
       and photo.identifier = photo_version.photo
       and files.version = photo_version.identifier");

  $size = pg_fetch_row($file_sizes);

  return $size[0];
}

function dirsize($database, $directory) {
  $result = array("size" => 0, "files" => 0);

  $query = pg_query($database, "select count(identifier), sum(filesize) from files where path LIKE '$directory%'");

  if ($query && pg_num_rows($query)) {
    $row = pg_fetch_row($query);
    $result['files'] = $row[0];
    $result['size'] = $row[1];
  }

  return $result;
}

function get_current_volume($database, $image_file) {
  global $image_repository_path;
  global $po_options_default;

  $curr_volume = readlink($image_repository_path."/current");
  $new_volume = $curr_volume;

  if (substr($curr_volume, 0, 1) == '/') {
    $curr_volume = substr($curr_volume, strlen($image_repository_path));
  }

  do {
    $dir_size = dirsize($database, $new_volume);

    if (is_file($image_file)) {
      $image_size = filesize($image_file);
    } else {
      $image_size = 0;
    }

    $dir_size['size'] = round($dir_size['size']/1048576, 2);

    /* if size is smaller return the volume as current volume */
    if (($dir_size['size'] + $image_size/1048576) < $po_options_default['volume_max_size'])
      break;

    if ($po_options_default['volume_max_count'] && 
	($po_options_default['volume_max_count'] < $dir_size['files']))
      break;

    $new_volume++;
    if (($directory_handle = @opendir($image_repository_path."/".$new_volume))) {
      /* if the next volume check if there is enough space */
      closedir($directory_handle);
      continue;
    } else {
      /* if the next volume does not exist, try to create it */
      if (!mkdir($image_repository_path."/".$new_volume, 0700))
        return FALSE;
      if (!mkdir($image_repository_path."/".$new_volume."/00000", 0700))
        return FALSE;
      if (!mkdir($image_repository_path."/".$new_volume."/00001", 0700))
        return FALSE;
      break;
    }
  } while (TRUE);

  if ($new_volume != $curr_volume) {
    unlink($image_repository_path."/current");
    po_symlink($new_volume, "$image_repository_path/current");
  }

  /* Make sure we pass back an absolute path */
  if (substr($new_volume, 0, 1) != '/') {
    $new_volume = "$image_repository_path/$new_volume";
  }

  return $new_volume;
}

function extract_exiftool_metadata($image_data, $index, $type = "EXIF") {
  global $sys_exiftool;
  global $strings;

  $image_data["exiftool_data_$type"] = NULL;
  $exif_data = array();
  $file = $image_data['file'][$index]['name_tmp'];
  $exclude_args = "";

  switch ($type) {
  case 'EXIF' :
    $exclude_args = "-x xmp:all -x iptc:all";
    break;
  case 'IPTC' :
    $exclude_args = "-iptc:all";
    break;
  case 'XMP' :
    $exclude_args = "-xmp:all";
    if (isset($image_data['sidecar'])) {
      $file = $image_data['sidecar'];
    }
    break;
  case 'ORIENT' :
    $exclude_args = "-Orientation -ICCProfile -ColorSpace -ProfileClass -Make -Model";
    break;
  }

  print "<li>".$strings['import']['extracting_metadata']." ($type) ... ";

  if (!is_executable($sys_exiftool)) {
    print err_str($strings['errors']['exiftool_not_installed']) ."</li>";
    return $image_data;
  }

  $handle = popen("$sys_exiftool $exclude_args " . escapeshellarg($file), "r");

  if ($handle === FALSE) {
    print err_str($strings['errors']['exiftool_failed']) ."</li>";
    return $image_data;
  }

  while (!feof($handle)) {
    $line = fgets($handle);
    $parts = explode(":", $line, 2);
    $parts[0] = trim($parts[0]);
    if (!isset($parts[1])) continue;
    $parts[1] = trim($parts[1]);
    if ($parts[0] == "File Name") continue;
    if ($parts[0] == "Directory") continue;
    if (substr($parts[1], 2, 6) == "Binary") continue;
    //  print "<li>key: $parts[0] value: $parts[1]</li>";
    $exif_data[$parts[0]] = $parts[1];
  }
  pclose($handle);
  $image_data["exiftool_data_$type"] = $exif_data;

  print $strings['generic']['done']."</li>";

  return $image_data;
}

function transfer_metadata($source_name, $dest_name, $verbose, $fix_orient) {
  global $sys_exiftool;
  global $po_options;
  global $strings;

  $retval = FALSE;

  /* Copy over image tags */
  if (is_executable($sys_exiftool)) {
    if ($verbose)
      print "<li>".$strings['import']['transferring_metadata']." ... ";
    
    if ($fix_orient) {
      $orient = " -Orientation=1 -n ";
    } else {
      $orient = "";
    }

    $cmdline = "$sys_exiftool -q -overwrite_original -TagsFromFile " . escapeshellarg($source_name) ." $orient ". escapeshellarg($dest_name);
    system($cmdline, $retval);

    if (!$retval) {
      if ($verbose) {
	print $strings['generic']['done'].".</li>";
      }
    } else {
      if ($verbose) {
	print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
	print "<li><pre>$cmdline</pre></li>";
      }
    }
  }
  return !$retval;
}

function photo_import($database, $index, $user_id, &$image_data) {
  global $mime_type;
  global $po_options;
  global $strings;
  global $image_repository_path;
  global $po_user;

  $result = TRUE;
  $cleanup_files = array();

  if (($current_volume = get_current_volume($database, $image_data['file'][$index]['name_tmp'])) == FALSE) {
    print err_str($strings['errors']['failed_volume']);
    return FALSE;
  }

  if (!is_readable($image_data['file'][$index]['name_tmp'])) {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  $cleanup_files[] = $image_data['file'][$index]['name_tmp'];

  /* Trims the temporary directory name that is used during bulk upload */
  $pos = strpos($image_data['file'][$index]['name'], "/");
  if (is_integer($pos)) {
    $original_name = substr($image_data['file'][$index]['name'], strrpos($image_data['file'][$index]['name'], "/") + 1);
  } else {
    $original_name = $image_data['file'][$index]['name'];
  }

  if ($original_name == '') {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  /* Check ownership of folder */
  $target_folder = pg_fetch_row(pg_query($database, "select caption, users from folder where identifier = $image_data[folder]"));
  if (($po_user['type'] != PO_USER_TYPE_ADMIN) &&
      ($target_folder[1] != $user_id)) {
    print err_str($strings['errors']['not_folder_owner']." (".emit_a(generate_link('folder', $image_data['folder']), $target_folder[0]). ")");
    return FALSE;
  }

  pg_query($database, "begin");

  $original_file_type                 = strtolower($image_data['file'][$index]['type']);

  $next_index = pg_fetch_row(pg_query($database, "select nextval('photo_id_sequence')"));

  $hires_name = $current_volume . "/00000/" . $next_index[0] . "_0_orig." . $original_file_type;

  $image_data['file'][$index]['original_name'] = $original_name;
  $multi_page_parameter               = $mime_type[$original_file_type]['page'];
  $mime_type_name                     = $mime_type[$original_file_type]['name'];
  $decoder                            = $mime_type[$original_file_type]['decoder'];

  $cleanup_files[] = $hires_name;

  print "<p>".sprintf($strings['import']['importing_into_folder'], $original_name, $mime_type_name, emit_a(generate_link('folder', $image_data['folder']), "<strong>$target_folder[0]</strong>"))."</p>";

  print "<ul class=\"import\">";

  /* Check quotas */
  if ($result && ($po_options['quota_size'] > 0)) {
    $used_storage_space = count_storage_space_by_user($database, $user_id);
    $used_storage_space += filesize($image_data['file'][$index]['name_tmp']);
    if ($po_options['quota_size'] <= $used_storage_space) {
      $result = FALSE;
      $po_options['quota_size'] /= 1048576;
      print "<li>";
      print err_str($strings['errors']['quota_space_full'] . " ($po_options[quota_size] MB)");
      print $strings['import']['no_more_uploads'];
      print "</li>";
    }
  }

  if ($result && ($po_options['quota_count'] > 0)) {
    $num_of_photo_versions = pg_fetch_row(pg_query($database, "select count_photo_versions_by_user($user_id)"));
    $num_of_photo_versions++;
    if ($po_options['quota_count'] <= $num_of_photo_versions[0]) {
      $result = FALSE;
      print "<li>";
      print err_str($strings['errors']['quota_count_full'] . " ($po_options[quota_count] ".$strings['generic']['files'] .")");
      print $strings['import']['no_more_uploads'];
      print "</li>";
    }
  }

  if ($result) {
    /* Extract any metadata */
    $types = explode(',', $image_data['import_metadata_order']);
    foreach ($types as $type) {
      switch (strtolower($type)) {
      case 'exif':
	$image_data = photo_parse_exif($database, $index, $user_id, $image_data);
	break;
      case 'xmp':
	$image_data = photo_parse_rdf($database, $index, $user_id, $image_data);
	break;
      case 'iptc':
	$image_data = photo_parse_iptc($database, $index, $user_id, $image_data);
	break;
      }
    }
  }

  /* Figure out orientation and necessary transforms */
  if ($result) {
    $orientation = orientation_id_from_string($database, $image_data["orientation"]); 
    $image_data["orientation_xform"] = orientation_to_xform($database, $orientation);
    
    $next_version = pg_fetch_row(pg_query($database, "select nextval('photo_version_id_sequence')"));
  }

  if ($result) {
    /* Sanity-check the timestamp */
    $image_data['date_of_exposure'] = check_date_validity($image_data['date_of_exposure']);

    /* Insert into database */
    $query = "insert into photo
            (identifier, users, folder, location,
             caption, date_of_exposure, access_rights, copyright_statement,
             views, hide_original, author, title,
             caption_writer, category, credit, source,
             headline, instructions, transmission_reference,
             supplemental_category, web_statement, store_url, comments)
     values ($next_index[0], '$user_id', '$image_data[folder]', $image_data[location],
             '$image_data[caption]', $image_data[date_of_exposure], '$image_data[access_rights]', '$image_data[copyright]',
             0, '$image_data[hide_original]', '$image_data[author]', '$image_data[title]', 
             '$image_data[caption_writer]', '$image_data[category]', '$image_data[credit]', '$image_data[source]',
             '$image_data[headline]', '$image_data[instructions]', '$image_data[transmission_reference]',
             '$image_data[supplemental_category]', '$image_data[web_statement]', '$image_data[store_url]', '$image_data[remark]')";
    $result = pg_query($database, $query);
  }

  if ($result) {
    $original_name = pg_escape_string($original_name);
    $comment = $image_data['file'][$index]['remark'];
    $orientation = orientation_id_from_string($database, $image_data["orientation"]); 
    $query = "insert into photo_version(identifier, key, photo, master, original_image_name, comment, colorspace, orientation)
                                   values ($next_version[0], 1,  $next_index[0], 't', '$original_name', '$comment', " . $image_data['colorspace'] . " , $orientation )";
    $result = pg_query($database, $query);
  } 

  /* Now for the keywords */
  if ($image_data['keywords']) {
    foreach ($image_data['keywords'] as $keyword) {
      if ($result) {
	$keyword = strtolower(trim($keyword));
	$keyword = trim($keyword, ','); /* Trailing commas */
	$keyword = trim($keyword);
	if ($keyword == "") continue;
	$keyword = pg_escape_string($keyword);
	
	$query = "insert into photo_keywords (photo, keyword) 
                   values ($next_index[0], '$keyword')";
	$result = pg_query($database, $query);
      }
    }
  }

  if ($result) {
    /* Deal with original file */
    $result = import_orig_file($image_data, $index, $hires_name, 
			       $decoder, $database, $next_version[0]);
  }
  
  /* Generate scaled images */
  if ($result) {
    $result = import_generate_all_scaled($image_data['file'][$index]['name_tmp'], $multi_page_parameter, $image_data["orientation_xform"], $image_data['colorspace'], $hires_name, TRUE, $database, $next_version[0], '', '', $current_volume, $next_index[0], $cleanup_files, $hires_name);
  
  }

  /* Now for the XMP sidecar, if present */
  if (isset($image_data['sidecar'])) {
    $xmp_file = $current_volume . "/00001/$next_index[0]_$next_version[0]_0_orig.xmp";
    if (!copy($image_data['sidecar'], $xmp_file)) {
      $result = FALSE;
      echo "<li><b>can't copy $image_data[sidecar] to $xmp_file</b></li>";
    }
    if ($result) {
      $cleanup_files[] = $xmp_file;
      $result = import_file($database, $xmp_file, $image_repository_path, $next_version[0], -1, FALSE, "", "");
      if (!$result) {
	echo "<li>file import blew up</li>";
      }
      unlink($image_data['sidecar']);
      unset($image_data['sidecar']);
    }
  }

  if ($result) {
    if ($image_data['storage_location_type'] == "null")
      $image_data['storage_location_type'] = "";

    $storage_location = $image_data['storage_location_type'].$image_data['storage_location_id'];
    $query = "insert into photo_dupe (identifier, key, photo, storage_location, storage_index, remark, version)
                                   values (nextval('photo_dupe_id_sequence'), 1, $next_index[0], '$storage_location', '$image_data[storage_index]', 'Original', $next_version[0])";
    $result = pg_query($database, $query);
  }
 
  if ($result) {
    if ($image_data['latitude'] == "") $image_data['latitude'] = "null";
    if ($image_data['longitude'] == "") $image_data['longitude'] = "null";
    if ($image_data['altitude'] == "") $image_data['altitude'] = "null";
    if ($image_data['direction'] == "") $image_data['direction'] = "null";
    
    $query = "insert into photo_tech (identifier, photo, camera, film, scan_resolution, aperture, focal_length, shutter, camera_metering, camera_program, flash_mode, exposure_comp, flash_comp, ev_difference, iso_override, scan_bitdepth, scan_multiscan, exif, iptc, latitude, longitude, altitude, img_direction, rdf)
        values (nextval('photo_tech_id_sequence'), $next_index[0], $image_data[camera],
                $image_data[film], $image_data[scan_resolution], $image_data[aperture],
                $image_data[focal_length], $image_data[shutter], $image_data[camera_metering],
                $image_data[camera_program], $image_data[flash_mode], $image_data[exp_comp],
                $image_data[flash_comp], $image_data[exp_diff], $image_data[iso_override],
                $image_data[scan_bitdepth], $image_data[scan_multiscan], 
                '$image_data[exif]', '$image_data[iptc]', $image_data[latitude], $image_data[longitude], $image_data[altitude], $image_data[direction], '$image_data[rdf]')";
    $result = pg_query($database, $query);
  }

  /* Take care of equipment tables */
  if ($image_data['equipment'])
  foreach ($image_data['equipment'] as $equip) {
    $parts = explode(":", $equip);
    $parts[0] = pg_escape_string($parts[0]);
    $parts[1] = pg_escape_string($parts[1]);
    if ($result) {
      $query = "insert into photo_equipment (photo, equipment, type) VALUES ($next_index[0], $parts[0], $parts[1])";
      $result = pg_query($database, $query);
    }
  }

  if ($result) {
    pg_query($database, "commit");
    $err = error_reporting(0);
    unlink($image_data['file'][$index]['name_tmp']); /* We're done with it */
    error_reporting($err);
    print "<li>".$strings['import']['complete']." (". emit_a(generate_link('photo', $next_index[0]), "#" . $next_index[0]) .")</li>";
  } else {
      if ($query != "") {
        $error = pg_last_error($database);
        print "<li>" . err_str($error) ."</li>";
        print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
        print "<li><pre>$query</pre></li>\n";
      }
      pg_query($database, "rollback");
      print "<li>".$strings['import']['cleaning_up']."</li>";
      $err = error_reporting(0);
      foreach ($cleanup_files as $file) {
        unlink($file);
      }
      error_reporting($err);      
  }

  print "</ul>";
  if (!$result) {
    return FALSE;
  }

  /* If we have an embedded preview image, import it as a secondary version */
  if ($image_data['jpgfromraw']) {
    $image_data_b = $image_data;

    /* Transfer metadata over to preview image */
    transfer_metadata($hires_name, 
		      $image_data['jpgfromraw'], FALSE, FALSE);

    /* Fill in the blanks */
    $image_data_b['file'][$index]['name']     = $image_data['file'][$index]['name'] . "_jpgfromraw.jpg";
    $image_data_b['file'][$index]['original_name']     = $image_data['file'][$index]['name'] . "_jpgfromraw.jpg";
    $image_data_b['file'][$index]['name_tmp'] = $image_data['jpgfromraw'];
    $image_data_b['file'][$index]['type']     = "jpeg";
    $image_data_b['file'][$index]['remark']   = "";
    $image_data_b['file'][$index]['master']   = $po_options['jpgfromraw_master'];
    $image_data_b['orientation'] = $image_data['orientation_orig'];
    $image_data_b['file'][$index]['remark']   = "Preview image extracted from RAW";
    $image_data_b['replace_existing'] = 'f';

    /* And import it.  This will unlink the temp file */
    $version_id = photo_version_import($database, $index, $user_id, $next_index[0], $image_data_b);
    if (!$version_id) {
      $err = error_reporting(0);
      unlink($image_data['jpgfromraw']);
      error_reporting($err);      
    }
  }
  
  return $next_index[0];
}

function photo_version_import($database, $index, $user_id, $photo_id, $image_data) {
  global $mime_type;
  global $po_options;
  global $strings;
  global $image_repository_path;
  global $po_user;

  $old_master = 0;
  $result = TRUE;
  $query = '';
  $cleanup_files = array();

  if (($current_volume = get_current_volume($database, $image_data['file'][$index]['name_tmp'])) == FALSE) {
    print err_str($strings['errors']['failed_volume']);
    return FALSE;
  }

  if (!is_readable( $image_data['file'][$index]['name_tmp'])) {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  $cleanup_files[] = $image_data['file'][$index]['name_tmp'];

  /* Trims the temporary directory name that is used during bulk upload */

  $pos = strpos($image_data['file'][$index]['name'], "/");
  if (is_integer($pos)) {
    $original_name = substr($image_data['file'][$index]['name'], strrpos($image_data['file'][$index]['name'], "/") + 1);
  } else {
    $original_name = $image_data['file'][$index]['name'];
  }

  $image_data['file'][$index]['original_name'] = $original_name;

  if ($original_name == '') {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  /* Check ownership of photo */
  $photo_data = pg_fetch_row(pg_query($database, "select caption, users, identifier from photo where photo.identifier = '$photo_id'"));
  if (($po_user['type'] != PO_USER_TYPE_ADMIN) &&
      ($po_user['id'] != $photo_data[1])) {
    print err_str($strings['errors']['not_owner']." (".emit_a(generate_link('photo', $photo_data[1]), "$photo_data[2] - $photo_data[0]"). ")");
    return FALSE;
  }

  pg_query($database, "begin");

  $original_file_type = strtolower($image_data['file'][$index]['type']);

  $next_index = pg_fetch_row(pg_query($database, "select nextval('photo_id_sequence')"));

  $hires_name = $current_volume . "/00000/" . $next_index[0] . "_3_orig." . $original_file_type;

  $multi_page_parameter               = $mime_type[$original_file_type]['page'];
  $mime_type_name                     = $mime_type[$original_file_type]['name'];
  $decoder                            = $mime_type[$original_file_type]['decoder'];

  $cleanup_files[] = $hires_name;

  print "<p>". sprintf($strings['import']['importing_new_version'], $original_name, $mime_type_name, emit_a(generate_link('photo', $photo_id, array('detail_info'=>5)), "<strong>$photo_id</strong>")) ."</p>";

  print "<ul class=\"import\">";

  /* Check quotas */  
  if ($result && ($po_options['quota_size'] > 0)) {
    $used_storage_space = count_storage_space_by_user($database, $user_id);
    $used_storage_space += filesize($image_data['file'][$index]['name_tmp']);
    if ($po_options['quota_size'] <= $used_storage_space) {
      $result = FALSE;
      $po_options['quota_size'] /= 1048576;
      print "<li>";
      print err_str($strings['errors']['quota_space_full'] . " ($po_options[quota_size] MB)");
      print $strings['import']['no_more_uploads'];
      print "</li>";
    }
  }

  if ($result && ($po_options['quota_count'] > 0)) {
    $num_of_photo_versions = pg_fetch_row(pg_query($database, "select count_photo_versions_by_user($user_id)"));
    $num_of_photo_versions++;
    if ($po_options['quota_count'] <= $num_of_photo_versions[0]) {
      $result = FALSE;
      print "<li>";
      print err_str($strings['errors']['quota_count_full'] . " ($po_options[quota_count] ".$strings['generic']['files'] .")");
      print $strings['import']['no_more_uploads'];
      print "</li>";
    }
  }

  /* Orientation and colorspace */
  if ($result) {
    $image_data = extract_exiftool_metadata($image_data, $index, "ORIENT");
    $image_data = photo_parse_exif_orientation_colorspace($database, $index, $user_id, $image_data, "ORIENT");
    
    $orientation = orientation_id_from_string($database, $image_data['orientation']);
    $colorspace = $image_data['colorspace'] == "" ? "1" : $image_data['colorspace'];
    
    $master = $image_data['file'][$index]['master'];

    $next_version = pg_fetch_row(pg_query($database, "select nextval('photo_version_id_sequence')"));
  }

  if ($result && ($master == 't')) {
    if ($image_data['replace_existing'] == 't') {
      $data = pg_fetch_row(pg_query("select version from photo_version where photo = $photo_id and master = 't'"));
      $old_master = $data[0];
    }
  
    $query = "update photo_version set master='f' where photo=$photo_id";
    $result = pg_query($database, $query);
  }

  if ($result) {
    $key = pg_fetch_row(pg_query($database, "select max(key) from photo_version where photo=$photo_id"));
    $key[0]++;
    
    $comment = $image_data['file'][$index]['remark'];
    $pos = strpos($image_data['file'][$index]['name'], "/");
    if (is_integer($pos)) {
      $original_name = substr($image_data['file'][$index]['name'], strrpos($image_data['file'][$index]['name'], "/") + 1);
    } else {
      $original_name = $image_data['file'][$index]['name'];
    }
    
    $query = "insert into photo_version (identifier, key, photo, master, original_image_name, comment, colorspace, orientation)
              values ($next_version[0], '$key[0]', '$photo_id', '$master', '$original_name', '$comment', $colorspace, $orientation)";
    $result = pg_query($database, $query);
  }

  if ($result) {
    $result = import_orig_file($image_data, $index, $hires_name, 
			       $decoder, $database, $next_version[0]);
  }

  if ($result && ($old_master != $next_version[0])) {
    $result = pg_query($database, "update album_content set version = $next_version[0] where photo = $photo_id and version = $old_master");
  }

  /* Generate scaled images */
  if ($result) {
    $result = import_generate_all_scaled($image_data['file'][$index]['name_tmp'], $multi_page_parameter, $image_data["orientation_xform"], $image_data['colorspace'], $hires_name, TRUE, $database, $next_version[0], '', '', $current_volume, $next_index[0], $cleanup_files, $hires_name);
  }

  if ($result) {
    pg_query($database, "commit");
    $err = error_reporting(0);
    unlink($image_data['file'][$index]['name_tmp']); /* We're done with it */
    error_reporting($err);
    print "</ul>";
    return $next_version[0];
  } else {
    if ($query != "") {
      print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
      print "<li><pre>$query</pre></li>\n";
    }
    print "<li>".$strings['import']['cleaning_up']."</li>";
    pg_query($database, "rollback");
    $err = error_reporting(0);
    foreach ($cleanup_files as $file) {
      unlink($file);
    }
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }
}

function import_raw_decode_dcraw($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  global $sys_dcraw;
  global $tmp_volume_path;
  global $strings;
  global $po_options;

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".ppm";

  if (!is_executable($sys_dcraw)) 
    return FALSE;

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'dcraw')." ... ";
  }

  /* Default to highest quality, and sRGB colorspace output */
  $options = " -q 3 -o 1";

  if ($po_options['dcraw_brightness'] != 1) {
    $options .= " -b $po_options[dcraw_brightness]";
  }

  if ($po_options['raw_denoise'] > 0) {
    $options .= " -n $po_options[raw_denoise]";
  }

  switch ($po_options['raw_white_balance']) {
  case "automatic":
    $options .= " -a";
    break;
  case "camera":
    $options .= " -w";
    break;
  }

  if ($po_options['raw_bitdepth'] == 48) {
    $options .= " -4";
  }

  if ($camera_input_profile != FALSE) {
    if ($camera_input_profile == "") $camera_input_profile = "embed";
    $options = $options . " -p $camera_input_profile";
  }

  $cmdline = "$sys_dcraw $options -c " . escapeshellarg($input_file_name) . " > $temporary_file_name ";

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  system($cmdline, $retval_decoding);

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
    return FALSE;
  }

  /* dcraw automatically rotates images */
  $image_data["orientation"] = "Normal (O deg)";

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
  }

  return $temporary_file_name;
}

function import_raw_decode_exiftool($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  global $sys_exiftool;
  global $tmp_volume_path;
  global $strings;

  if (!is_executable($sys_exiftool))
    return FALSE;

  /* If we don't have a RAW preview image, we're SOL here */
  if (!$image_data['jpgfromraw'])
    return FALSE;
  
  /* If we're already extracted the image, just use that one. */
  if (file_exists($image_data['jpgfromraw']))
    return $image_data['jpgfromraw'];

  /* Clear the jpegfromraw data so we don't think we have a secondary
     image to import.  If we actually do, the caller has to fix it up. */
  $image_data['jpgfromraw'] = FALSE;

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".jpg";

  $options = " -b -JpgFromRaw > " . escapeshellarg($temporary_file_name);

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'ExifTool')." ... ";
  }

  $cmdline = "$sys_exiftool " . escapeshellarg($input_file_name) . " $options ";

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  system($cmdline , $retval_decoding);

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
    return FALSE;
  }

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
  }

  return $temporary_file_name;
}

function import_raw_decode_ufraw($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  // global $icc_profiles;
  global $sys_ufraw;
  global $po_options;
  global $tmp_volume_path;
  global $strings;

  if (!is_executable($sys_ufraw))
    return FALSE;

  $options = "";

  if ($camera_input_profile != FALSE && $camera_input_profile != "") {
    $temporary_config_file = tempnam($tmp_volume_path, "po");
       $output_file=fopen($temporary_config_file,"w");
       fwrite($output_file,"<?xml version='1.0' encoding='utf-8'?>\n");
       fwrite($output_file,"<UFRaw Version='7'>\n");
       fwrite($output_file,"<InputProfile Current='yes'>Profile\n");
       fwrite($output_file,"   <File>".$camera_input_profile."</File>\n");
       fwrite($output_file,"</InputProfile>\n");
       fwrite($output_file,"<sRGBOutputProfile Current='yes'>\n");
       fwrite($output_file,"</sRGBOutputProfile>\n");
       fwrite($output_file,"</UFRaw>\n");
       fclose($output_file);
    $options = $options . " --conf=$temporary_config_file";
  }

  $curve = "camera"; // curve == linear/camera/filename
  $exposure = "auto"; // exposure compensation

  $linearity = $po_options['ufraw_gamma_linearity'];
  $gamma = $po_options['ufraw_gamma']; 

  if (($curve == "camera") || ($curve == "linear") || ($curve=="custom")) { 
    $curve = "--base-curve=$curve";
  } else {
    $curve = "--base-curve-file=$curve";
  }

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".ppm";

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'ufraw')." ... ";
  }

  if ($po_options['raw_denoise'] > 0) {
    $options .= " --wavelet-denoising-threshold=$po_options[raw_denoise]";
  }

  $options .= " --exposure=$exposure --gamma=$gamma --linearity=$linearity  --interpolation=ahd $curve";
  switch ($po_options['raw_white_balance']) {
  case "automatic":
    $options .= " --wb=auto";
    break;
  case "camera":
    $options .= " --wb=camera";
  }
  switch ($po_options['raw_bitdepth']) {
  case 24: 
    $options .= " --out-type=ppm8";
    break;
  case 48:
    $options .= " --out-type=ppm16";
  }

  $cmdline = "$sys_ufraw $options --silent --output=$temporary_file_name " . escapeshellarg($input_file_name); 

  system($cmdline, $retval_decoding);

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
       unlink($temporary_config_file);

    return FALSE;
  }

  /* ufraw automatically rotates images */
  $image_data["orientation"] = "Normal (O deg)";

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
  }

  unlink($temporary_config_file);

  return $temporary_file_name;
}

function import_raw_decode_dcm2pnm($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  global $sys_dcm2pnm;
  global $tmp_volume_path;
  global $strings;

  if (!is_executable($sys_dcm2pnm))
    return FALSE;

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".pnm";

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'dcm2pnm')." ... ";
  }

  $options = "";

  switch ($po_options['raw_bitdepth']) {
  case 24: 
    $options = $options . " --write-8-bit-pnm";
    break;
  case 48:
    $options = $options . " --write-16-bit-pnm";
  }

  $cmdline = "$sys_dcm2pnm $options " . escapeshellarg($input_file_name) ." $temporary_file_name ";

  system($cmdline, $retval_decoding);

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
    return FALSE;
  }

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
  }

  return $temporary_file_name;
}

function import_raw_decode($input_file_name, $decoder, $camera_input_profile, $verbose, &$image_data) {

  global $icc_profiles;

  if (($camera_input_profile == FALSE) ||
      ($camera_input_profile == null) ||
      ($camera_input_profile == "None")) {
    $camera_input_profile = FALSE;
  } else {
    $camera_input_profile = $icc_profiles[$camera_input_profile]['file'];
  }
  
  /* Figure out if we have an embedded preview image to extract */
  if ($image_data['jpgfromraw']) {
    /* This extracts the embedded image to a temporary file */
    $image_data['jpgfromraw'] = import_raw_decode_exiftool($input_file_name, $camera_input_profile, $verbose, $image_data);
  } 

  $res = FALSE;
  $decoders = split(",", $decoder);
  foreach ($decoders as $decoder) {
    $decoder = trim($decoder);
    if ($decoder == "dcraw") {
      $res = import_raw_decode_dcraw($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else if ($decoder == "exiftool") {
      $res = import_raw_decode_exiftool($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else if ($decoder == "ufraw") {
      $res = import_raw_decode_ufraw($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else if ($decoder == "dcm2pnm") {
      $res = import_raw_decode_dcm2pnm($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else {
      $res = FALSE;
    }  
    if ($res != FALSE) break;
  }
  return $res;
}

function import_generate_scaled($input_file_name, $output_file_name, 
				$multi_page_parameter, $size_key, 
				$orientation, $colorspace, 
				$original_file_name, $verbose,
				$database, $version_id, $params, $comments, $master_file_name)
{
  global $sys_convert;
  global $sys_composite;
  global $sys_gm;
  global $use_gm;
  global $icc_profiles;
  global $strings;
  global $po_options;
  global $image_repository_path; 

  $watermark = FALSE;

  /* Resize */
  switch ($size_key) {
  case 'thumb':
    $size_name = 'thumb';
    if (!$use_gm) {
      $resize_cmd = ' -thumbnail ';
    }
    $resize_cmd = ' -resize ';
    $resize_cmd .= escapeshellarg($po_options[$size_key.'_max_size'] ."x". $po_options[$size_key.'_max_size']) ."\> ";
    break;
  case 'full':
    # no resizing.
    $size_name = 'preview';
    break;
  default:
    $size_name = 'preview';
    $resize_cmd = ' -resize ';
    $resize_cmd .= escapeshellarg($po_options[$size_key.'_max_size'] ."x". $po_options[$size_key.'_max_size']) ."\> ";
    break;
  }

  /* matting options */
  if (($po_options[$size_key.'_border_width'] != 0) && ($po_options[$size_key.'_border_height'] != 0))
    $im_framing_option = " -frame " . escapeshellarg($po_options[$size_key.'_border_width']  ."x". $po_options[$size_key.'_border_height']) . " -mattecolor " . escapeshellarg("#".$po_options[$size_key.'_border_color']);

  else
    $im_framing_option = "";

  /* ICC transformation options */
  $im_profile_a = $im_profile_b = "";

  if ($colorspace != "" && $colorspace > "1") {
    $im_profile_b = " -profile " . $icc_profiles[1]['file'] . " +profile icm";

    if ($colorspace != "99") {
      $im_profile_a = " +profile icm -profile " . $icc_profiles[$colorspace]['file'];
    }
  }

  
  if ($verbose) {
    print "<li>".$strings['import']['generating_'.$size_name] ." ... ";
  }

  /* generate preview */
  if ($use_gm) {
    $cmdline = "$sys_gm convert";
  } else {
    $cmdline = "$sys_convert ";
  }
  $cmdline .=
    $im_profile_a .
    $resize_cmd .
    " -unsharp ".     escapeshellarg($po_options[$size_key.'_sharpen_radius']  ."x". $po_options[$size_key.'_sharpen_sigma'] ."+". $po_options[$size_key.'_sharpen_amount'] ."+". $po_options[$size_key.'_sharpen_threshold']) ." ".
    $im_framing_option ." ".
    $orientation ." ".
    $im_profile_b ." ".
    (($po_options[$size_key.'_color_space'] == "RGB") ? "" : " -colorspace " . escapeshellarg($po_options[$size_key.'_color_space']))  ." ".
    (($po_options[$size_key.'_gamma'] == 1) ? "" : " -gamma " .      escapeshellarg($po_options[$size_key.'_gamma'])) .
    " -quality " .  escapeshellarg($po_options[$size_key.'_quality']) ." ".
    escapeshellarg($input_file_name).escapeshellarg($multi_page_parameter) . ' ' .
    ($use_gm ? '' : ' -strip ') . 
    escapeshellarg($output_file_name);
  
  /*
  if ($verbose) {
    print "<em>$cmdline</em>";
  }
  */
  system($cmdline, $retval_image_scaling);

  if ($verbose) {
    print $strings['generic']['done'] . " (". $po_options[$size_key.'_max_size'] ." ".$strings['generic']['pixels']. ")</li>";
  }
  
  /* Are we supposed to watermark the image? */
  if ($po_options[$size_key.'_watermark'] &&
      $po_options['watermark_location'] && 
      $po_options['watermark_brightness'] && 
      $po_options['watermark_photo']) {
    
    $watermark = $po_options['watermark_photo'];

    /* Fix up the watermark path as needed */
    if (substr($po_options['watermark_path'], 0, 1) != '/') {
      $po_options['watermark_path'] = "$image_repository_path/$po_options[watermark_path]";
    }
    
    if ($verbose) {
      print "<li>".$strings['import']['watermarking'] ." ... ";
    }

    if ($use_gm) {
      $cmdline = "$sys_gm composite ";
    } else {
      $cmdline = "$sys_composite ";
    }
    $cmdline .=
      " -gravity " .  escapeshellarg($po_options['watermark_location']) .
      " -watermark " .  escapeshellarg($po_options['watermark_brightness']) ." ".
      escapeshellarg($po_options['watermark_path']) ." ".
      escapeshellarg($output_file_name)." ".
      " -quality " .  escapeshellarg($po_options[$size_key.'_quality']) ." ".
      escapeshellarg($output_file_name);

    /*
    if ($verbose) {
      print "<em>$cmdline</em>";
    }
    */

    system($cmdline, $retval_watermarking);
        
    if ($retval_watermarking) {
      if ($verbose) {
	print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
	print "<li><pre>$cmdline</pre></li>";
      }
      return FALSE;
    }

    if ($verbose) {
      print $strings['generic']['done'].".</li>";
    }
  }

  $result = TRUE;

  if ($po_options[$size_key.'_copy_metadata'] == 't') {
    $result = transfer_metadata($master_file_name, $output_file_name, $verbose, TRUE);
  }

  if ($result) {
    $result = import_file($database, $output_file_name, $image_repository_path,
			  $version_id, $po_options[$size_key.'_sizecode'],
			  $watermark, $params, $comments);
  }

  if ($verbose) {
    flush();
  }

  return $result;
}

/* Copies the original file into its resting place in the repository
   and performs a RAW decode if necessary */

function import_orig_file(&$image_data, $index, $hires_name, 
			  $decoder, $database, $version_id) {
  global $strings;
  global $po_options;
  global $image_repository_path;

  $success = TRUE;

  if ($image_data['file'][$index]['name_tmp']) {

    $original_name = $image_data['file'][$index]['original_name'];

    print "<li>" . sprintf($strings['import']['copying_into_repo'], $original_name) . " ... ";
    /* copy the original to the repository */
    if (!copy($image_data['file'][$index]['name_tmp'], $hires_name)) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>cp ".$image_data['file'][$index]['name_tmp']." $hires_name</pre></li>";
      return FALSE;
    }
    print $strings['generic']['done']."</li>";

    /* Raw Decode */
    $temporary_ppm_file = import_raw_decode($image_data['file'][$index]['name_tmp'], $decoder, $image_data['camera_input_profile'], TRUE, $image_data);
    if ($temporary_ppm_file) {
      unlink($image_data['file'][$index]['name_tmp']);
      $image_data['file'][$index]['name_tmp'] = $temporary_ppm_file;
    }

    /* raw decode can alter orientation */
    $orientation = orientation_id_from_string($database, $image_data["orientation"]);
    $image_data["orientation_xform"] = orientation_to_xform($database, $orientation);

    // print "<li>$image_data[orientation] => $orientation => $image_data[orientation_xform]</li>";

    /* Copy the original image into the repository */
    $success = import_file($database, $hires_name, $image_repository_path, 
			   $version_id, 0, FALSE, '', '');

  }

  return $success;
}

/* Inserts a file record into the database */
function import_file($database, $file_name, $base_path, $version_id, 
		     $size_code, $watermark, $params, $comments)
{
  global $query;

  if (!$watermark)
    $watermark = 'null';
  $params = pg_escape_string($params);
  $comments = pg_escape_string($comments);

  $sha1sum = sha1_file($file_name);
  if ($size_code < 0) {
    $image_dimension = array(0, 0);
  } else {
    $image_dimension = po_get_image_size($file_name);
  } 
  $size = filesize($file_name);
  $file = substr($file_name, strlen($base_path));
  if (substr($file, 0, 1) == '/') {
    $file = substr($file, 1);
  }

  $query = "INSERT INTO files 
     (identifier, version, size, x_res, y_res, path, filesize, params, comments, watermark, sha1sum) 
      VALUES ((select nextval('files_id_sequence')), $version_id, $size_code, 
              $image_dimension[0], $image_dimension[1], '$file', 
              $size, '$params', '$comments', $watermark, '$sha1sum')";
  $result = pg_query($database, $query);

  return $result;
}

function import_generate_all_scaled($input_file_name, $multi_page_parameter, 
				    $orientation, $colorspace, 
				    $original_file_name, $verbose,
				    $database, $version_id, $params, $comments,
				    $current_volume, $next_index, &$outfiles, $master_file_name)
{
  global $po_options;
  global $image_respository_path;

  $sizes = split(",", $po_options['scaled_generate']);

  foreach ($sizes as $size_key) {
    $output_file_name = "$current_volume/00001/$next_index" . "_$version_id" . "_" . $po_options[$size_key."_sizecode"] ."_$size_key." . $po_options[$size_key."_format"];

    $outfiles[] = $output_file_name;

    $result = import_generate_scaled($input_file_name, $output_file_name, 
				     $multi_page_parameter, $size_key, 
				     $orientation, $colorspace, 
				     $original_file_name, $verbose,
				     $database, $version_id, 
				     $params, $comments, $master_file_name);
  }

  return $result;
}

?>
