<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2008 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

function convert_iptc2sql_timestamp($date, $time) {
  $year   = substr($date,  0, 4);
  $month  = substr($date,  4, 2);
  $day    = substr($date,  6, 2);

  if ($date == "" || $year == 0 || $month == 0 || $day == 0) {
    $today = getdate();
    $year   = $today['year'];
    $month  = $today['mon'];
    $day    = $today['mday'];
  }

  $hour   = substr($time,  0, 2);
  $minute = substr($time,  2, 2);
  $second = substr($time,  4, 2);

  if ($date == "") {
    $hour   = "00";
    $minute = "00";
    $second = "00";
  }

  $sql_timestamp = $year.'-'.$month.'-'.$day.' '.$hour.':'.$minute.':'.$second;
  return $sql_timestamp;
}

function convert_iptc_location($database, $user_id, $place, $city, $state, $country) {
  $location = pg_fetch_row(pg_query($database, "
     select location.identifier
       from location, country, state
      where '$city' ~*  location.city
        and '$state' ~* state.value
        and location.state = state.identifier
        and '$country' ~*  country.value
        and '$place' ~* location.place
        and location.country = country.identifier"));

  /* unlike elsewhere we look for exact matches */
  if ($location) {
    return $location[0];
  }

  /* if no location found create a new one in the database */
  pg_query($database, "begin");

  /* check for existing state/region */
  /* if not found create one */
  if (($state) && ($state != 'null')) {
    $state_id = pg_fetch_row(pg_query($database, "select identifier from state where value ~* '$state'"));
    if (!$state_id) {
      $state_id = pg_fetch_row(pg_query($database, "select nextval('state_id_sequence')"));
      if (!$state_id) {
        pg_query($database, "rollback");
        return "0";
      }
      pg_query($database, "insert into state (identifier, value) values ($state_id[0], '$state')");
    }
  }

  /* check for existing country */
  /* if not found create one - you never know these days */
  if (($country) && ($country != 'null')) {
    $country_id = pg_fetch_row(pg_query($database, "select identifier from country where value ~* '$country'"));
    if (!$country_id) {
      $country_id = pg_fetch_row(pg_query($database, "select nextval('country_id_sequence')"));
      if (!$country_id) {
        pg_query($database, "rollback");
        return "0";
      }
      pg_query($database, "insert into country (identifier, value) values ($country_id[0], '$country')");
      // print "insert into country (identifier, value) values ($country_id[0], '$country'";
    }
  }

  $new_location_id = pg_fetch_row(pg_query($database, "select nextval('location_id_sequence')"));
  if (!$new_location_id) {
    pg_query($database, "rollback");
    return "0";
  }

  $state_id[0] = $state_id[0] ? $state_id[0] : "null";
  $country_id[0] = $country_id[0] ? $country_id[0] : "null";

  $result = pg_query($database, "insert into location
                                        (identifier, place, city, state, country, last_modified_date, users, private)
                                 values ($new_location_id[0], '$place', '$city', $state_id[0], $country_id[0], now(), $user_id, 'f')");
  if (!$result) {
    pg_query($database, "rollback");
    return "0";
  }

  pg_query($database, "commit");

  return $new_location_id[0];
}

function weird_to_extended_ascii($s) {
  $conversion_table = array(
                            128 => "",
                            129 => "",
                            130 => "",
                            131 => "",
                            132 => "",
                            133 => "",
                            134 => "",
                            135 => "",
                            136 => "",
                            137 => "",
                            138 => "",
                            139 => "",
                            140 => "",
                            141 => "",
                            142 => "",
                            143 => "",
                            144 => "",
                            145 => "",
                            146 => "",
                            147 => "",
                            148 => "",
                            149 => "",
                            150 => "",
                            151 => "",
                            152 => "",
                            153 => "",
                            154 => "",
                            155 => "",
                            156 => "",
                            157 => "",
                            158 => "",
                            159 => "",
                            163 => "",
                            164 => "",
                            166 => "",
                            167 => "",
                            168 => "",
                            169 => "",
                            174 => "",
                            175 => "",
                            180 => "",
                            181 => "",
                            190 => "",
                            191 => "",
                            192 => "",
                            203 => "",
                            204 => "",
                            205 => "",
                            206 => "",
                            207 => "",
                            229 => "",
                            230 => "",
                            231 => "",
                            232 => "",
                            233 => "",
                            234 => "",
                            235 => "",
                            236 => "",
                            237 => "",
                            238 => "",
                            239 => "",
                            241 => "",
                            242 => "",
                            243 => "",
                            244 => ""
                            );

  $len = strlen($s);
  for($x = 0; $x < $len; $x++) {
    if (ord($s[$x]) < 128) {
      $result .= $s[$x];
    } else {
      if ($conversion_table[ord($s[$x])]) {
        $result .= $conversion_table[ord($s[$x])];
      }
      else {
        $result .= ord($s[$x]);
      }
    }
  }
  return $result;
}

function validate_iptc_string($iptc_string) {
  $string_length = strlen($iptc_string);
  $newstring = "";

  /* Remove non displayable ASCII characters */
  for ($i=0; $i < $string_length; $i++) {
    $ascii_num = ord($iptc_string[$i]);

    if ($ascii_num == 0x27)
      continue;

    if ((0xff > $ascii_num) && ($ascii_num > 0x1F))
      $newstring = $newstring . $iptc_string[$i];
  }

  $newstring = weird_to_extended_ascii($newstring);

  /* Check for null fields. All strings containing spaces only are */
  /* condsidered null, just like dates of format 0000:00:00 00:00:00 */
  /* therefore if a string contains only zero, column or space */
  /* return an empty string */

  $trans = array(" " => "", ":" => "", "-" => "", "0" => "");
  $stripped_string = strtr($newstring, $trans);
  if ($stripped_string == "")
    return $stripped_string;

  return $newstring;
}

function photo_parse_iptc($database, $index, $user_id, $image_data) {
  global $strings;

  $place = "";
  $city = "";
  $state = "";
  $country = "";
  $creation_date = "";
  $creation_time = "";
  $keywords = "";

  $image_data = extract_exiftool_metadata($image_data, $index, "IPTC");
  /* Populate IPTC data */
  $iptc_data = $image_data['exiftool_data_IPTC'];
  foreach ($iptc_data as $key => $value) {
    $image_data["iptc"] .= "<key>$key</key><value>$value</value>";
    switch ($key) {
    case "Caption-Abstract":
      $image_data["caption"] = $value;
      break;
    case "Writer-Editor":
      $image_data["caption_writer"] = $value;
      break;
    case "By-line":
      $image_data["author"] = $value;
      break;
    case "Copyright Notice":
      $image_data["copyright"] = $value;
      break;
    case "Object Name":
      $image_data["title"] = $value;
      break;
    case "Category":
      $image_data["category"] = $value;
      break;
    case "Supplemental Categories":
      $image_data["supplemental_category"] = $value; // add to keywords?
      break;
    case "Special Instructions":
      $image_data["instructions"] = $value;
      break;
    case "Headline":
      $image_data["headline"] = $value;
      break;
    case "Credit":
      $image_data["credit"] = $value;
      break;
    case "Source":
      $image_data["source"] = $value;
      break;
    case "Original Transmission Reference":
      $image_data["transmission_reference"] = $value;
      break;
    case "City":
      $city = $value;
      break;
    case "Province-State":
      $state = $value;
      break;
    case "Country-Primary Location Name":
      $country = $value;
      break;
    case "Sub-location":
      $place = $value;
      break;
    case "Creation Date":
      $creation_date = $value;
      break;
    case "Creation Time":
      $creation_time = $value;
      break;
    case "Keywords":
      $keywords = $value;
      break;
    case "Application Record Version":
      // Ignore
      break;
    default:
      // print "<li>UNHANDLED: $key = $value</li>";
      break;
    }
  }

  if (($place != "") || ($city != "") || ($state != "") || ($country != "")) {
    $image_data['location'] = convert_iptc_location($database, $user_id, $place, $city, $state, $country);
  }

  if (($creation_date != "") || ($creation_time != "")) {
    $image_data['date_of_exposure'] = convert_iptc2sql_timestamp($creation_date, $creation_time);
  }

  if ($keywords != "") {
    $keys = explode(", ", $keywords);
    foreach ($keys as $key) {
      if (!in_array($key, $image_data['keywords']))
	$image_data['keywords'][] = $key;
    }
  }

  /* Escape it for database sanity */
  $image_data["iptc"] = pg_escape_string($image_data["iptc"]);

  return $image_data;
}

?>
