<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2008 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "config.php";
include_once "common.php";
include_once "vars.php";

/* Simple database connection wrapper */
function po_dbconnect() {
  global $db_dsn;
  
  $database = pg_pconnect($db_dsn);   /* Persistent */
  if ($database)
    pg_set_client_encoding($database, 'UTF-8');

 return $database;
}

function site_prolog($min_type = PO_USER_TYPE_DISABLED) {
  global $po_user;
  global $po_options;
  global $po_options_default;
  global $default_cred_timeout;
  global $po_session_lifetime;
  global $po_session_path;
  global $po_session_handler;
  global $po_cookie_path;
  global $strings;
  global $generate_i18n_data;
  global $passwords;
  global $po_memcache;
  global $memcache;

  $sybase = FALSE;
  // if magic quotes is enabled
  if ( get_magic_quotes_gpc () ) {
    // workaround because magic_quotes does not change $_SERVER['argv']
    $argv = isset($_SERVER['argv']) ? $_SERVER['argv'] : NULL; 
    
    // fix all affected arrays
    foreach ( array ('_ENV', '_REQUEST', '_GET', '_POST', '_COOKIE', '_SERVER') as $var ) {
      $GLOBALS[$var] = fix_magic_quotes ($GLOBALS[$var], $sybase);
    }
    $_SERVER['argv'] = $argv;
    
    // turn off magic quotes, this is so scripts which
    // are sensitive to the setting will work correctly
    ini_set ('magic_quotes_gpc', 0);
  }
  
  set_magic_quotes_runtime(0);
  
  /* Turn off error display */
  ini_set ('display_errors', 0);
  
  /* Establish DB Connection */
  $database = po_dbconnect();

  if ($database == FALSE) return FALSE;

  /* Connect to the memcache */
  if ($po_memcache) {
    $memcache = new Memcache;
    $memcache->connect($po_memcache);
  }

  /* Start up session */
  if ($po_session_handler != "") 
	ini_set("session.save_handler", $po_session_handler);
  if ($po_session_path != "") 
	ini_set("session.save_path", $po_session_path);
  if ($po_session_lifetime) {
	ini_set("session.cache_expire", $po_session_lifetime / 60);
	ini_set("session.gc_maxlifetime", "$po_session_lifetime");
  }
  session_name("po_session");
  session_start();

  /* See if user is logged in; Set up session data.. */
  if (isset($_SESSION['po_user'])) {
    $new_user = $_SESSION['po_user'];

    foreach ($new_user as $key => $value) {
      $po_user[$key] = $new_user[$key];
    }

    /* Set last request time */
    $po_user['last_request'] = time();
  }

  $po_user['session_id'] = session_id();

  /* And then set up option data */
  if (isset($_SESSION['po_options'])) {
    $options = $_SESSION['po_options'];
    
    foreach ($options as $key => $value) {
      $po_options[$key] = $options[$key];
    }
  }

  /* Periodically refresh our cached credentials & preferences */
  if (($po_user['id'] != 0) && 
      ($po_user['last_updated'] + $default_cred_timeout) < time()) {
    /* Update preferences and credentials */
    refresh_user_prefs($database);
  }

  $tmp = array_merge($po_user['passwords'], $po_options['passwords']);
  $passwords = pg_escape_string(implode("," , $tmp));

  /* See if request updated things any -- Don't store! */
  if (isset($_REQUEST['order_by']) && isset($_REQUEST['order_by_req'])) {
    $po_options['order_by'] = $_REQUEST['order_by'];
  }
  if (isset($_REQUEST['folder_order_by']) && isset($_REQUEST['order_by_req'])) {
    $po_options['folder_order_by'] = $_REQUEST['folder_order_by'];
    if ($memcache) {
      $memcache->delete("allalbum_$po_user[id]");
      $memcache->delete("allfolder_$po_user[id]");
    }
  }
  if (isset($_REQUEST['photo_view'])) {
    $po_options['photo_view'] = $_REQUEST['photo_view'];
  }

  if (isset($_REQUEST['photos_per_page']) && isset($_REQUEST['per_page_req'])) {
    $po_options['photos_per_page'] = $_REQUEST['photos_per_page'];
  }
  if (isset($_REQUEST['rows_list_view']) && isset($_REQUEST['per_page_req'])) {
    $po_options['rows_list_view'] = $_REQUEST['rows_list_view'];
  }

  /* If a language tag is set, use it. */
  if (isset($_REQUEST['po_lang']))
    $po_options['lang'] = $_REQUEST['po_lang'];

  if (isset($_REQUEST['po_theme']))
    $po_options['theme'] = $_REQUEST['po_theme'];

  if ($memcache) {
    $user_data = array('last_name' => $po_user['last_name'],
		       'first_name' => $po_user['first_name'],
		       'username' => $po_user['username']);
    $memcache->set("user_data_$po_user[id]", $user_data);
  }

  /* Update session details. */
  site_update_session(FALSE);

  /* And now, close it out to release locks */
  session_write_close();

  /* Sanity check in case the cookie gets screwed up */
  if (($min_type == PO_USER_TYPE_DISABLED) && !$po_user['type']) {
    session_start();
    setcookie(session_name(), '', time()-3600, $po_cookie_path);
    session_destroy();    
  }
  
  /* Ensure user has minimum permissions for this page */
  if ($po_user['type'] < $min_type) {
    switch ($min_type) {
    case PO_USER_TYPE_ADMIN:
      site_permission_error('admin_perm');
      break;
    case PO_USER_TYPE_USER:
      site_permission_error('user_perm');
      break;
    case PO_USER_TYPE_CLIENT:
      site_permission_error('client_perm');
      break;
    default:
      site_permission_error('insuff_perm');
      break;
    }
    site_epilog($database);
    exit();
  }

  /* Set up the locale settings and whatnot */
  global $dayNames;
  global $monthNames;
  include "lang/".$po_options['lang'].".php";

  /* Set up the theme */
  global $site_url;
  global $site_title;
  global $site_title;
  global $thm_elem;
  global $theme;
  $theme = "themes/".$po_options['theme'];
  if (!is_readable("$theme/theme.php"))
    $theme = "themes/".$po_options_default['theme'];
  include "$theme/theme.php";

  foreach ($generate_i18n_data as $fn) {
	$fn();
  }

  setlocale(LC_ALL, $strings['formats']['locale_names']);
  $foo = localeconv();
  $strings['formats']['decimal'] = $foo['decimal_point'];
  $strings['formats']['thousands'] = $foo['thousands_sep'];
  setlocale(LC_NUMERIC, 'POSIX', 'C');
    
  return $database;
}

function setup_user_info($database, $auth_handle, $username, $lang, $auto_login) {
  global $po_user;
  global $po_options;

  /* Get the user info from the auth system */
  $user_data = $auth_handle->user_info($username);

  if ($auth_handle->local_userinfo) {
    /* Now time to fetch the rest of the crap */
    global $database;

    $foo = pg_fetch_assoc(pg_query($database, "select address1, address2, city, zipcode, state, country, phone, url from users where username = '$username'"));
    if ($foo)
      $user_data = array_merge($user_data, $foo);
  }

  /* Assign a few more variables */
  $po_user['username'] = $username;
  $po_user['auto_login'] = ($auto_login != "");
  $po_options['lang'] = $lang;

  /* Look up PO's internal user ID */
  $res = pg_query($database, "SELECT identifier FROM users WHERE username='$username'");
  if (($res != FALSE) && pg_num_rows($res)) {
    $row = pg_fetch_assoc($res);
    $po_user['id'] = $row['identifier'];
  } else { /* Auth valid, but user not in database.  register them properly, then log them in.. */
    $po_user['id'] = register_user($database, $username, $user_data['type'], "",
				   $user_data['first_name'], $user_data['last_name'], 
				   $user_data['email'], $auth_handle);
  }

  foreach ($user_data as $key => $value) {
    $po_user[$key] = $value;
  }

  /* Update local DB if the auth is external */
  if (!$auth_handle->local_register) {
    pg_query($database, "update users set first_name = '".pg_escape_string($po_user['first_name']).
	     "', last_name = '".pg_escape_string($po_user['last_name']).
	     "', email='".pg_escape_string($po_user['email']).
	     "', type=$po_user[type]
	         where identifier = $po_user[id]");
  }

  /* Update local DB if we don't use local userinfo */
  if (!$auth_handle->local_userinfo) {
    pg_query($database, "update users set company='".pg_escape_string($po_user['company']).
	     "', address1='".pg_escape_string($po_user['address1']).
	     "', address2='".pg_escape_string($po_user['address2']).
	     "', city='".pg_escape_string($po_user['city']).
	     "', zipcode='".pg_escape_string($po_user['zipcode']).
	     "', phone='".pg_escape_string($po_user['phone']).
	     "', url='".pg_escape_string($po_user['url']).
	     "' where identifier = $po_user[id]");

    if ($po_user['state']) {
      pg_query($database, "update users set state=(select identifier from state where value ='$po_user[state]') where identifier = $po_user[id]");
    }
    if ($po_user['country']) {
      pg_query($database, "update users set country=(select identifier from country where value ='$po_user[country]') where identifier = $po_user[id]");
    }
  }

  refresh_user_prefs($database);
  site_update_session();
  
  if ($po_user['type'] == PO_USER_TYPE_DISABLED) {
    $reason = "disabled_account";
    return FALSE;
  }

  return TRUE;
}

/* Updates the session state to match current configuration */
function site_update_session($start_sess=TRUE) {
  global $po_options;
  global $po_user;
  global $po_cookie_path;

  if ($po_user['session_id']) 
    session_id($po_user['session_id']);

  if ($start_sess) 
    session_start();

  if ($po_user['auto_login']) {
    $res = setcookie(session_name(), session_id(), time()+157680000, $po_cookie_path);
    // site_push_error("auto_login = $po_user[auto_login], cookie = $res, sess = $po_user[session_id], start_sess = $start_sess");
    $po_user['auto_login'] = FALSE;
  }

  $_SESSION['po_options'] = $po_options;
  $_SESSION['po_user'] = $po_user;  

  if ($start_sess)
    session_write_close();
}

function site_epilog($database) {
  pg_close($database);
}

/* Deal with errors */

function site_push_error($error) {
  global $po_user;

  session_id($po_user['session_id']);

  session_start();
  $_SESSION['errors'][] = $error;
  session_write_close();
}

function site_dump_errors() {
  global $po_user;

  session_id($po_user['session_id']);
  session_start();

  if (isset($_SESSION['errors']) && count($_SESSION['errors']) > 0) {
    print "<div class=\"errorbar\">\n";
    print "<ul>\n";
    foreach ($_SESSION['errors'] as $error) {
      print "<li>$error</li>\n";
    }
    print "</ul>\n";
    print "</div>\n";
  }

  $_SESSION['errors'] = array();
  session_write_close();
}

function err_str($error) {
  return "<font color=\"red\">$error</font>";
}

function site_referer_error($reason) {
  global $strings;

  if (isset($_SERVER['HTTP_REFERER'])) {
    site_push_error($strings['errors'][$reason]);
    header("Location: $_SERVER[HTTP_REFERER]");
  } else {
    site_permission_error($reason);
  }
}

function site_permission_error($reason, $url = FALSE) {
  global $strings;

  site_push_error($strings['errors'][$reason]);
  if ($url === FALSE) {
    $url = $_SERVER['REQUEST_URI'];
  }
  $url = urlencode($url);
  header("Location: login.php?reason=$reason&orig=$url");
}

/* Stuff that spits out HTML */

function site_adv() {
  global $ad_script;

  if ($ad_script) {
    print "<div class=\"banner_adv\">";
    print $ad_script;
    print "</div>";
  }
}

function site_logo() {
  global $thm_elem;

  print "<div id=\"site_logo\">$thm_elem[site_logo]</div>\n";

}

function site_credit() {
  global $strings;
  global $po_version;

  print "<div class=\"site_credit\">";

  print $strings['generic']['po_powered_by'] ." v$po_version<br/>\n";
  print $strings['generic']['po_copyright'];
  
  print "</div>\n";
}

function site_head($page_title, $title_b = "") {
  global $compress_pages;
  global $site_title;
  global $site_url;
  global $thm_elem;
  global $strings;
  global $pretty_urls;

  if ($compress_pages &&
      ini_get('zlib.output_compression') == '' &&
      ini_get('zend_accelerator.compress_all') == '' &&
      ini_get('output_handler') != 'ob_gzhandler') {
    if (ob_get_level()) {
      ob_end_clean();
    }
    ob_start('ob_gzhandler');
  }

  header("Content-Type: text/html; charset=".$strings['formats']['encoding']);
/*
  header("Content-Type: application/xhtml+xml");
  print "<?xml version=\"1.0\" encoding=\"".$strings['formats']['encoding']."\"?>\n";
*/
  print "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">";
  print "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"".$strings['formats']['lang_xml']."\" lang=\"".$strings['formats']['lang_xml']."\" dir=\"".$strings['formats']['direction']."\">";
  print "<head>\n";

  if ($pretty_urls)
    print "<base href=\"$site_url\" />";

  print "<title>$site_title : $page_title". ($title_b != "" ? " : $title_b" : "") ."</title>\n";
  print "<!-- compliance patch for microsoft browsers -->\n";
  print "<!--[if lt IE 8]><script src=\"ie7/ie7-standard-p.js\" type=\"text/javascript\"></script><![endif]-->\n";
  print "<!--[if lt IE 8]><script src=\"ie7/ie7-css-strict.js\" type=\"text/javascript\"></script><![endif]-->\n";

  //  print "<!--[if lt IE 8]><script src=\"javascript/firebug/firebug.js\" type=\"text/javascript\"></script><![endif]-->\n";

  print "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=".$strings['formats']['encoding']."\" />";

  print $thm_elem['shortcut_icon'];
  print $thm_elem['stylesheets'];
  print $thm_elem['javascript'];

  print "</head>\n";
}

function site_header($page_title, $title_b = "") {
  global $po_user;
  global $po_options;
  global $install_enabled;
  global $strings;

  site_head($page_title, $title_b);

  print "<body>\n";

  print "<script type=\"text/javascript\" src=\"javascript/wz_tooltip.js\"></script>\n";
  
  if ($install_enabled) {
    site_push_error($strings['errors']['install_enabled']);
  }

  if (isset($_SERVER['HTTP_USER_AGENT']) && 
      strstr($_SERVER['HTTP_USER_AGENT'], 'FunWebProducts')) {
	site_push_error("You are using the FunWebProducts toolbar.  
Aside from being considered spyware/adware, it hijacks your 
browser and sends malformed requests to the server, resulting in 404s or 
other errors.  Your experience on this site (and others) will improve 
considerably if you disable or uninstall this toolbar. See 
http://www.tnet.com/fun-web-products.php for further details.");
}

  if (isset($po_user['admin_change_password'])) {
    site_push_error($strings['errors']['default_admin']);
  }

  site_dump_errors();

  site_logo();
  if ($po_options['show_ads'] != 'f') {
    site_adv();
  }

  print "<div id=\"content\">\n";
}

function site_foot() {
  global $ad_script2;

  if ($ad_script2) {
    print $ad_script2;
  }

  print "</body></html>\n";
}

function site_footer($database) {
  print "</div> <!-- id=content --> \n";

  site_credit();
  site_foot();
}

function site_navigator_status($left_string, $right_string) {
  print "<!-- Navigator Status Begin -->\n";
  print "<div class=\"status\">";
  print "<table><tr>";
  print "<td>";
  print "$left_string\n";
  print "</td><td>";
  print "$right_string\n";
  print "</td></tr></table>";
  print "</div>";
  print "<!-- Navigator Status End -->\n";
}

function site_search_entry($size) {
  global $strings;
  global $thm_elem;

  print "<form action=\"search.text.general.php\" method=\"post\" accept-charset=\"".$strings['formats']['encoding']."\" >";
  print "<table align=\"center\" border=\"0\">";
  print "<tr><td>";
  print "<input type=\"text\" name=\"search_string\" value=\"$search_string\" size=\"$size\" title=\"".$strings['search']['search_string_prompt']."/>";
  print "</td><td>";
  print $thm_elem['button.search'];
  print "</td></tr>";
  print "</table>";
  print "</form>";
}

/* These are sort of meant to be called externally .. */

function site_display_photo_b($photo_id, $link = TRUE, $size = 1, $version = FALSE, $target = FALSE)
{
  $image = generate_link('image', $photo_id, array('ver'=>$version, 'size'=>$size));

  if (!$target) {
    $target = generate_link('photo', $photo_id, array('ver'=>$version));
  } 

  if ($link == TRUE) {
    print emit_a($target, "<img src=\"$image\" alt=\"$photo_id:$version\" />", "class=\"nodecor\"");
  } else {
    print "<img src=\"$image\" alt=\"$photo_id:$version\" />";
  }
}

function site_display_photo($database, $identifier, $framed, $version = FALSE) {
  if ($version === FALSE) {
    $version = "photo_version.master = 't'";
  } else {
    $version = "photo_version.identifier = '$version' ";
  }

  $filter = " photo.identifier = '$identifier' and photo_version.photo = photo.identifier and $version ";

  $photo_data = get_photo_query($database, array(), array(), $filter, 0, 0, 21);

  if (pg_num_rows($photo_data) < 1)
    return;

  display_photo_slides("folder", "", $photo_data, 0, 1, $framed, FALSE);
}

function site_get_photos($database, $type, $identifier, $num_of_photos, $order) {
  $sql_extra_tables = array();

  switch ($type) {
  case "photo":
    $parts = explode(":", $identifier);
    $parts[0] = pg_escape_string($parts[0]);
    $sql_photo_selector = "photo.identifier = '$parts[0]' ";
    if (count($parts) > 1) {
      $parts[1] = pg_escape_string($parts[1]);
      $sql_photo_selector .= " and photo_version.identifier = '$parts[1]'";
    } else {
      $sql_photo_selector .= " and photo_version.master = 't' ";
    } 
    break;
  case "user":
    $sql_photo_selector = "photo.users = '$identifier' and photo_version.master = 't' ";
    break;
  case "album":
    $sql_photo_selector = "photo_version.identifier = album_content.version and photo.identifier = album_content.photo and album_content.album=".$identifier;
    $sql_extra_tables[] = "album_content";
    break;
  default:
  case "folder":
    $sql_photo_selector = "photo.folder = '$identifier' and photo_version.master = 't' ";
    break;
  }
  
  $filter = " $sql_photo_selector ";

  $photo_data = get_photo_query($database, array(), $sql_extra_tables, $filter, 0, $num_of_photos, $order);
  return $photo_data;
}

function site_display_random_photos($database, $type, $identifier, $num_of_photos, $framed) {
  $photo_data = site_get_random_photos($database, $type, $identifier, $num_of_photos, 21);
  display_photo_slides($type, $identifier, $photo_data, 0, pg_num_rows($photo_data), $framed, FALSE);
}

function site_display_user_name($database, $identifier, $link) {
  print disp_user_string($database, $identifier, $link);
}

?>
