
//  Copyright (C) 2005 Balint Kis (balint@k-i-s.net)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.applet.*;
import java.applet.Applet;
import java.net.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.accessibility.*;

class BButton {
    int offsetX, offsetY;
    Dimension dimension;
    boolean isPressed;
    boolean isHighLighted;
    Image image[];
    Applet applet;

    public BButton(Applet applet, int offsetX, int offsetY, Dimension dimension, String fileName) {
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.dimension = dimension;
        this.isPressed = false;
        this.isHighLighted = false;
        this.applet = applet;

        image = new Image[4];
        image[0] = applet.getImage(applet.getDocumentBase(), fileName + ".normal.gif");
        image[1] = applet.getImage(applet.getDocumentBase(), fileName + ".pressed.gif");
        image[2] = applet.getImage(applet.getDocumentBase(), fileName + ".highlighted.gif");
        image[3] = applet.getImage(applet.getDocumentBase(), fileName + ".disabled.gif");
    }

    public boolean isInFocus(int cx, int cy) {
        return ((cx > offsetX) && (cx < (offsetX + dimension.width)) &&
                (cy > offsetY) && (cy < (offsetY + dimension.height)));
    }

    public void setHighLight(boolean isHighLighted) {
        this.isHighLighted = isHighLighted;
        if (!isHighLighted)
            isPressed = false;
    }

    public void setStatus(boolean isPressed) {
        this.isPressed = isPressed;
    }

    public void paint(Graphics g) {
        if (isPressed) {
            g.drawImage(image[1], offsetX, offsetY, applet);
            return;
        }
        if (isHighLighted) {
            g.drawImage(image[2], offsetX, offsetY, applet);
            return;
        }
        g.drawImage(image[0], offsetX, offsetY, applet);
    }
}

class ColorChooser {
    int offsetX, offsetY;
    int markerX, markerY;
    Dimension dimension;
    Applet applet;

    private int[] colorMap;
    private Image imageMap = null;

    public ColorChooser(Applet applet, int offsetX, int offsetY, Dimension dimension){
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.dimension = dimension;
        this.applet = applet;

        colorMap = new int[dimension.width * dimension.height];
        for (int x = 0; x < dimension.width; x++){
            for (int y = 1; y <= dimension.height; y++){
                float H = ((float) x)*((float)1/(float)dimension.width);
                float S = 1;
                if (y > (float)dimension.height/(float)2) {
                    S = (1-((float)y/(float)dimension.height))*(float)2;
                }
                float B = 1;
                if (y < (float)dimension.height/(float)2) {
                    B = ((float)y/(float)dimension.height)*(float)2;
                }
                colorMap[x + (dimension.height - y) * dimension.width] = Color.HSBtoRGB(H, S, B);
            }
        }

        imageMap = applet.createImage(new MemoryImageSource(dimension.width, dimension.height, colorMap, 0, dimension.width));
    }

    public void paint(Graphics g) {
        g.drawImage(imageMap, offsetX, offsetY, applet);
        g.setColor(Color.gray);
        g.drawLine(offsetX, offsetY, offsetX, offsetY + dimension.height);
        g.drawLine(offsetX, offsetY + dimension.height, offsetX + dimension.width, offsetY + dimension.height);
        g.drawLine(offsetX + dimension.width, offsetY + dimension.height, offsetX + dimension.width, offsetY);
        g.drawLine(offsetX + dimension.width, offsetY, offsetX, offsetY);

        g.setColor(Color.orange);
        g.drawLine(offsetX + markerX - 11, offsetY + markerY - 1, offsetX + markerX + 11, offsetY + markerY - 1);
        g.drawLine(offsetX + markerX - 11, offsetY + markerY,     offsetX + markerX + 11, offsetY + markerY);
        g.drawLine(offsetX + markerX - 11, offsetY + markerY + 1, offsetX + markerX + 11, offsetY + markerY + 1);
        g.drawLine(offsetX + markerX - 1, offsetY + markerY - 11, offsetX + markerX - 1, offsetY + markerY + 11);
        g.drawLine(offsetX + markerX,     offsetY + markerY - 11, offsetX + markerX,     offsetY + markerY + 11);
        g.drawLine(offsetX + markerX + 1, offsetY + markerY - 11, offsetX + markerX + 1, offsetY + markerY + 11);

        g.setColor(Color.black);
        g.drawLine(offsetX + markerX - 10, offsetY + markerY, offsetX + markerX + 10, offsetY + markerY);
        g.drawLine(offsetX + markerX, offsetY + markerY - 10, offsetX + markerX, offsetY + markerY + 10);
    }

    public boolean isInFocus(int cx, int cy) {
        return ((cx > offsetX) && (cx < (offsetX + dimension.width)) &&
                (cy > offsetY) && (cy < (offsetY + dimension.height)));
    }

    public Color getColor(int x, int y){
        markerX = x - offsetX;
        markerY = y - offsetY;
        return new Color(colorMap[markerX + markerY * dimension.width]);
    }

//     public void setColorMarker(Color color){
//         for (int x = 0; x < dimension.width; x++){
//             for (int y = 1; y <= dimension.height; y++){
//                 float H = ((float)x/(float)dimension.width);
//                 float S = 1;
//                 if (y > (float)dimension.height/(float)2) {
//                     S = (1-((float)y/(float)dimension.height))*(float)2;
//                 }
//                 float B = 1;
//                 if (y < (float)dimension.height/(float)2) {
//                     B = ((float)y/(float)dimension.height)*(float)2;
//                 }
//                 if (color == new Color(Color.HSBtoRGB(H, S, B))) {
//                     markerX = x;
//                     markerY = y;
//                     return;
//                 }
//             }
//         }
//     }
}

class ColorStack {
    Color currentColor;
    Color colorStack[];

    int offsetX, offsetY;
    int selectedColorIndex;
    int currentColorIndex;
    int depth;
    Dimension dimension;

    ColorStack(int depth, int offsetX, int offsetY, Dimension dimension) {
        this.depth = depth;
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.dimension = dimension;

        /* create the initial color stack */
        colorStack = new Color[depth];

        for (int x = 0; x < depth; x++)
            colorStack[x] = null;

        currentColorIndex = depth - 1;
        selectedColorIndex = depth;
    }

    public void paint(Graphics g) {
        for (int x = 0; x < depth; x++) {
            if (colorStack[x] == null)
                continue;

            g.setColor(colorStack[x]);
            g.fillRect(offsetX, offsetY + x * dimension.height, dimension.width, dimension.height);
        }

        /* draw grids */
        g.setColor(Color.gray);
        g.drawLine(offsetX, offsetY, offsetX, offsetY + depth * dimension.height);
        for (int x = 0; x <= depth; x++)
            g.drawLine(offsetX, offsetY + x * dimension.height, offsetX + dimension.width, offsetY + x * dimension.height);
        g.drawLine(offsetX + dimension.width, offsetY, offsetX + dimension.width, offsetY + depth * dimension.height);

        /* draw marker */
        if (selectedColorIndex < depth) {
            g.setColor(Color.black);
            g.drawLine(offsetX, offsetY + (selectedColorIndex) * dimension.height, offsetX + dimension.width, offsetY + (selectedColorIndex) * dimension.height);
            g.drawLine(offsetX, offsetY + (selectedColorIndex + 1) * dimension.height, offsetX + dimension.width, offsetY + (selectedColorIndex + 1) * dimension.height);
            g.drawLine(offsetX, offsetY + (selectedColorIndex) * dimension.height, offsetX, offsetY + (selectedColorIndex + 1) * dimension.height);
            g.drawLine(offsetX + dimension.width, offsetY + (selectedColorIndex) * dimension.height, offsetX + dimension.width, offsetY + (selectedColorIndex + 1) * dimension.height);

            g.setColor(Color.black);
            g.drawLine(offsetX - dimension.height,   offsetY + (selectedColorIndex)     * dimension.height,
                       offsetX - dimension.height,   offsetY + (selectedColorIndex + 1) * dimension.height);
            g.drawLine(offsetX - dimension.height,   offsetY + (selectedColorIndex)     * dimension.height,
                       offsetX - dimension.height/2, offsetY + (selectedColorIndex)     * dimension.height + dimension.height/2);
            g.drawLine(offsetX - dimension.height/2, offsetY + (selectedColorIndex)     * dimension.height + dimension.height/2,
                       offsetX - dimension.height,   offsetY + (selectedColorIndex + 1) * dimension.height);


            g.drawLine(offsetX + dimension.width + dimension.height,   offsetY + (selectedColorIndex)     * dimension.height,
                       offsetX + dimension.width + dimension.height,   offsetY + (selectedColorIndex + 1) * dimension.height);
            g.drawLine(offsetX + dimension.width + dimension.height,   offsetY + (selectedColorIndex)     * dimension.height,
                       offsetX + dimension.width + dimension.height/2, offsetY + (selectedColorIndex)     * dimension.height + dimension.height/2);
            g.drawLine(offsetX + dimension.width + dimension.height/2, offsetY + (selectedColorIndex)     * dimension.height + dimension.height/2,
                       offsetX + dimension.width + dimension.height,   offsetY + (selectedColorIndex + 1) * dimension.height);
        }
    }

    public boolean isInFocus(int cx, int cy) {
        /* mark selected item */
        for (int index = 0; index < depth; index++) {
            if ((cx > (offsetX)) && (cx < (offsetX + dimension.width)) &&
                (cy > (offsetY + index * dimension.height)) && (cy < (offsetY + (index + 1) * dimension.height))) {
                if (colorStack[index] != null) {
                    selectedColorIndex = index;
                    return true;
                }
            }
        }
        return false;
    }

    public Color getColor() {
        return colorStack[selectedColorIndex];
    }

    public void setCurrentColor(Color newColor) {
        currentColor = newColor;
    }

    public void insertNew() {
        if (currentColorIndex > -1) {
            colorStack[currentColorIndex] = currentColor;
            if (currentColorIndex == (depth - 1))
                selectedColorIndex = currentColorIndex;
            currentColorIndex--;
        }
    }

    public void clearCurrent() {
        for (int x = selectedColorIndex; x >= 1; x--)
            colorStack[x] = colorStack[x-1];

        colorStack[0] = null;

        currentColorIndex++;

        if (selectedColorIndex == currentColorIndex)
            selectedColorIndex++;
    }

    public void clearAll() {
        for (int x = 0; x < depth; x++)
            colorStack[x] = null;

        currentColorIndex = depth - 1;
        selectedColorIndex = depth;
    }

    public String getStackContentString() {
        int effectiveDepth = 0;
        for (int x = 0; x < depth; x++)
            if (colorStack[x] != null)
                effectiveDepth++;

        String parameters = "submit=yes" + "&width=" + effectiveDepth + "&height=" + 1;
        for (int x = 0; x < depth; x++)
            if (colorStack[depth - 1 - x] != null) {
                int red   = colorStack[depth - 1 - x].getRed();
                int green = colorStack[depth - 1 - x].getGreen();
                int blue  = colorStack[depth - 1 - x].getBlue();
                parameters = parameters + "&px" + x + "=" + Integer.toHexString(blue + 256 * (green + 256 * red));
            }
        return parameters;
    }
}

class ColorPreview {
    int offsetX, offsetY;
    Dimension dimension;
    Color color;
    Applet applet;

    public ColorPreview(Applet applet, int offsetX, int offsetY, Dimension dimension){
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.dimension = dimension;
        this.applet = applet;
        this.color = Color.white;
    }

    public void paint(Graphics g) {
        g.setColor(color);
        g.fillRect(offsetX, offsetY, dimension.width, dimension.height);
        g.setColor(Color.gray);
        g.drawLine(offsetX, offsetY, offsetX, offsetY + dimension.height);
        g.drawLine(offsetX, offsetY + dimension.height, offsetX + dimension.width, offsetY + dimension.height);
        g.drawLine(offsetX + dimension.width, offsetY + dimension.height, offsetX + dimension.width, offsetY);
        g.drawLine(offsetX + dimension.width, offsetY, offsetX, offsetY);
    }

    public void setPreviewColor(Color color) {
        this.color = color;
    }

    public Color getPreviewColor() {
        return color;
    }
}

public class ColorSearch extends Applet implements Runnable, MouseListener, MouseMotionListener {
    Thread       canvasPainter;
    Image        offScreen;
    Dimension    offScreensize;
    Graphics     offGraphics;

    Dimension    dimensionCanvas;
    ColorStack   colorStack;
    ColorPreview colorPreview;
    ColorChooser colorChooser;

    BButton buttonInsert;
    BButton buttonClear;
    BButton buttonClearAll;
    BButton buttonSubmit;

    String imagePath = null;
    String submitStringURL = null;
    URL    submitAnchorURL = null;

    int depth = 15;

    public void init() {
        String parameter;

        dimensionCanvas = new Dimension(400, 400);

        if ((parameter = getParameter("width")) != null)
            dimensionCanvas.width = Integer.parseInt(parameter);

        if ((parameter = getParameter("height")) != null)
            dimensionCanvas.height = Integer.parseInt(parameter);

        if ((imagePath = getParameter("image_path")) == null)
            imagePath = "";

        submitStringURL = getParameter("submit_url");
        if (submitStringURL != null) {
            try {
                submitAnchorURL = new URL(submitStringURL);
            } catch (MalformedURLException e) {
                System.err.println("ColorSearch: Parse error: " + e);
            }
        }

        colorChooser = new ColorChooser(this, 0, 0, new Dimension(dimensionCanvas.width - 160, dimensionCanvas.height));

        buttonClear    = new BButton     (this, dimensionCanvas.width - 130,   0, new Dimension(72, 25), getCodeBase() + imagePath + "/clear");
        buttonClearAll = new BButton     (this, dimensionCanvas.width - 130,  30, new Dimension(72, 25), getCodeBase() + imagePath + "/clear.all");
        colorPreview   = new ColorPreview(this, dimensionCanvas.width - 130,  60, new Dimension(72, 25));
        buttonInsert   = new BButton     (this, dimensionCanvas.width - 130,  90, new Dimension(72, 25), getCodeBase() + imagePath + "/insert");
        colorStack     = new ColorStack  (depth,dimensionCanvas.width - 130, 120, new Dimension(72, (dimensionCanvas.height - 151)/depth));
        buttonSubmit   = new BButton     (this, dimensionCanvas.width - 130, dimensionCanvas.height - 30, new Dimension(72, 25), getCodeBase() + imagePath + "/submit");

        addMouseListener(this);

        for (int index = 0; index < depth; index++) {
            String colorString = getParameter("px" + index);
            if ((colorString == null) || (colorString == ""))
                continue;
            colorString = colorString.replaceAll("#", "");
            Color currentColor = new Color(Integer.parseInt(colorString, 16));
            colorStack.setCurrentColor(currentColor);
            colorStack.insertNew();
        }
    }

    public void destroy() {
        removeMouseListener(this);
    }

    public void start() {
        canvasPainter = new Thread(this);
        canvasPainter.start();
    }

    public synchronized void stop() {
        canvasPainter = null;
        notify();
    }

    public void submitSearch(URL submitAnchorURL) {
        try {
            String parameters = colorStack.getStackContentString();
            if (parameters =="")
                return;

            URL submitURL = new URL(submitAnchorURL + "?" + parameters);
            getAppletContext().showDocument(submitURL);
        } catch (Exception e) {
            System.err.println("ColorSearch: HTTP Post Error: " + e);
        }
    }

    public void paint(Graphics g) {
    }

    public synchronized void startAnimation() {
        notify();
    }

    public void run() {
        Thread me = Thread.currentThread();
        while (canvasPainter == me) {
            repaint();
            try {
                Thread.sleep(100);
            }
            catch (InterruptedException e) {
                break;
            }
        }
    }

    public synchronized void update(Graphics graphics) {
        Dimension d = getSize();
        if ((offScreen == null) || (d.width != offScreensize.width) || (d.height != offScreensize.height)) {
            offScreen = createImage(d.width, d.height);
            offScreensize = d;
            offGraphics = offScreen.getGraphics();
            offGraphics.setFont(getFont());
        }
        offGraphics.setColor(Color.white);
        offGraphics.fillRect(0, 0, d.width, d.height);
        paint(offGraphics);
        buttonInsert.paint(offGraphics);
        buttonClear.paint(offGraphics);
        buttonClearAll.paint(offGraphics);
        buttonSubmit.paint(offGraphics);
        colorStack.paint(offGraphics);
        colorChooser.paint(offGraphics);
        colorPreview.paint(offGraphics);
        graphics.drawImage(offScreen, 0, 0, null);
    }

    public void mousePressed(MouseEvent e) {
        addMouseMotionListener(this);
        buttonInsert.setStatus(buttonInsert.isInFocus(e.getX(), e.getY()));
        buttonClear.setStatus(buttonClear.isInFocus(e.getX(), e.getY()));
        buttonClearAll.setStatus(buttonClearAll.isInFocus(e.getX(), e.getY()));
        buttonSubmit.setStatus(buttonSubmit.isInFocus(e.getX(), e.getY()));

        if (colorChooser.isInFocus(e.getX(), e.getY())) {
            Color currentColor;
            currentColor = colorChooser.getColor(e.getX(), e.getY());
            colorStack.setCurrentColor(currentColor);
            colorPreview.setPreviewColor(currentColor);
        }

        mouseClicked(e);
    }

    public void mouseDragged(MouseEvent e) {
        if (colorChooser.isInFocus(e.getX(), e.getY())) {
            Color currentColor;
            currentColor = colorChooser.getColor(e.getX(), e.getY());
            colorStack.setCurrentColor(currentColor);
            colorPreview.setPreviewColor(currentColor);
        }
        mouseClicked(e);
    }

    public void mouseClicked(MouseEvent e) {
        repaint();
        e.consume();
    }

    public void mouseReleased(MouseEvent e) {
        removeMouseMotionListener(this);

        if (buttonInsert.isInFocus(e.getX(), e.getY())) {
            buttonInsert.setStatus(false);
            Color currentColor = colorPreview.getPreviewColor();
            colorStack.setCurrentColor(currentColor);
            colorStack.insertNew();
        }
        if (buttonClear.isInFocus(e.getX(), e.getY())) {
            buttonClear.setStatus(false);
            colorStack.clearCurrent();
        }
        if (buttonClearAll.isInFocus(e.getX(), e.getY())) {
            buttonClearAll.setStatus(false);
            colorStack.clearAll();
        }
        if (buttonSubmit.isInFocus(e.getX(), e.getY())) {
            buttonSubmit.setStatus(false);
            if (submitAnchorURL != null) {
                submitSearch(submitAnchorURL);
            }
        }

        if (colorStack.isInFocus(e.getX(), e.getY())) {
            Color currentColor = colorStack.getColor();
//             colorChooser.setColorMarker(currentColor);
//             colorChooser.paint(offGraphics);

            colorPreview.setPreviewColor(currentColor);
        }
        repaint();
        e.consume();
    }

    public void mouseEntered(MouseEvent e) {
        mouseMoved(e);
    }

    public void mouseExited(MouseEvent e) {
        mouseMoved(e);
    }

    public void mouseMoved(MouseEvent e) {
        buttonInsert.setHighLight(buttonInsert.isInFocus(e.getX(), e.getY()));
        buttonClear.setHighLight(buttonClear.isInFocus(e.getX(), e.getY()));
        buttonClearAll.setHighLight(buttonClearAll.isInFocus(e.getX(), e.getY()));
        buttonSubmit.setHighLight(buttonSubmit.isInFocus(e.getX(), e.getY()));
        e.consume();
    }


    public String getAppletInfo() {
	return "Photo Organizer Color Search Client v1.0 \n Copyright (C) 2005, Balint Kis (balint@k-i-s.net).";
    }

    public String[][] getParameterInfo() {
	String[][] info = {
	  {"width",      "int", "width of the applet in pixels"},
	  {"height",     "int", "height of the applet in pixels"},
            {"image_path", "URL", "URL pointing to the images"},
            {"submit_url", "URL", "submit URL"}
	};
	return info;
    }
}
